package pps.core.base.service;

import cn.hutool.core.collection.CollUtil;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import org.apache.commons.lang3.StringUtils;
import pps.cloud.base.service.BasePowerLineCloudService;
import pps.cloud.base.service.data.base_power_line.DynamicQueryBasePowerLineInput;
import pps.cloud.base.service.data.base_power_line.DynamicQueryBasePowerLineOutput;
import pps.cloud.base.service.data.base_power_line.GetBasePowerLineInput;
import pps.cloud.base.service.data.base_power_line.GetBasePowerLineViewOutput;
import pps.core.base.entity.BasePowerLineEnt;
import pps.core.base.mapper.BasePowerLineMapper;
import pps.core.common.entity.BaseModel;
import xstartup.annotation.XService;
import xstartup.base.XContext;
import xstartup.base.util.XCopyUtils;
import xstartup.data.XListResult;
import xstartup.data.XSingleResult;

import java.util.List;

/**
 * 输电线路配置模块
 *
 * @author ZWT
 * @date 2023/09/05 15:37
 */
@XService
public class BasePowerLineCloudServiceImpl implements BasePowerLineCloudService {

    /**
     * 输电线路配置模块--通过ID查询线路基本信息
     *
     * @param context 上下文
     * @param input   输入
     * @return {@link XSingleResult}<{@link GetBasePowerLineViewOutput}>
     */
    @Override
    public XSingleResult<GetBasePowerLineViewOutput> getBasePowerLineById(XContext context, GetBasePowerLineInput input) {
        BasePowerLineMapper mapper = context.getBean(BasePowerLineMapper.class);
        BasePowerLineEnt entity = mapper.selectOne(new LambdaQueryWrapper<BasePowerLineEnt>()
                .eq(BaseModel::getId, input.getId())
        );
        GetBasePowerLineViewOutput output = XCopyUtils.copyNewObject(entity, GetBasePowerLineViewOutput.class);
        return XSingleResult.success(output);
    }

    /**
     * 输电线路配置模块--条件查询线路基本信息列表
     *
     * @param context 上下文
     * @param input   输入
     * @return {@link XListResult}<{@link DynamicQueryBasePowerLineOutput}>
     */
    @Override
    public XListResult<DynamicQueryBasePowerLineOutput> getBasePowerLineList(XContext context, DynamicQueryBasePowerLineInput input) {
        String lineId = input.getLineId();
        List<String> lineIds = input.getLineIds();
        String ouId = input.getOuId();
        String upperLineId = input.getUpperLineId();
        String lineName = input.getLineName();
        String gridTypeKey = input.getGridTypeKey();
        String lineTypeKey = input.getLineTypeKey();
        String strategyId = input.getStrategyId();
        BasePowerLineMapper mapper = context.getBean(BasePowerLineMapper.class);
        List<BasePowerLineEnt> selectList = mapper.selectList(new LambdaQueryWrapper<BasePowerLineEnt>()
                .select(BaseModel::getId, BasePowerLineEnt::getLineName)
                .in(CollUtil.isNotEmpty(lineIds), BaseModel::getId, lineIds)
                .eq(StringUtils.isNotBlank(lineId), BaseModel::getId, lineId)
                .eq(StringUtils.isNotBlank(ouId), BasePowerLineEnt::getOuId, ouId)
                .eq(StringUtils.isNotBlank(upperLineId), BasePowerLineEnt::getUpperLineId, upperLineId)
                .like(StringUtils.isNotBlank(lineName), BasePowerLineEnt::getLineName, lineName)
                .eq(StringUtils.isNotBlank(gridTypeKey), BasePowerLineEnt::getGridTypeKey, gridTypeKey)
                .eq(StringUtils.isNotBlank(lineTypeKey), BasePowerLineEnt::getLineTypeKey, lineTypeKey)
                .eq(StringUtils.isNotBlank(strategyId), BasePowerLineEnt::getStrategyId, strategyId)
                .orderByDesc(BaseModel::getModifyTime)
        );
        List<DynamicQueryBasePowerLineOutput> lineOutputList = XCopyUtils.copyNewList(selectList, DynamicQueryBasePowerLineOutput.class);
        return XListResult.success(lineOutputList);
    }
}